from abc import ABC, abstractmethod
import collections.abc
from dataclasses import dataclass
from typing import List, Any

import toml
import copy

import paperboat.pipeline

class Configuration:
    config: dict
    @classmethod
    def update(cls, d, u):
        """
        Update a dictionary with the "default values" from dictionary u
        :param d: the target dictionary (won't be modified)
        :param u: the values to update
        :return: the updated dictionary
        """
        # make a deep copy of d
        nd = copy.deepcopy(d)

        for k, v in u.items():
            if isinstance(v, collections.abc.Mapping):
                nd[k] = cls.update(nd.get(k, {}), v)
            else:
                nd[k] = v
        return nd

    def __init__(self, preset: dict = None):
        if preset == None:
            preset = dict()
        self.config = preset

    def set(self, setting_uri: str, value: Any):
        comp = setting_uri.split(".")
        d = self.config
        for c in comp[:-1]:
            if not c in d:
                d[c] = dict()
            d = d[c]

        d[comp[-1]] = value

    def set_dict(self, new_values: dict):
        self.config = Configuration.update(self.config, new_values)

    def get_dict(self) -> dict:
        return self.config



"""



@deserialize
@dataclass
class StepDescription:
    name: str
    config: dict

class StepConfiguration(ABC):
    pass

class StepGenerator(ABC):
    @abstractmethod
    def get_step(self, config: StepConfiguration) -> paperboat.pipeline.Step: pass




@deserialize
@dataclass
class ConfigurationFile:
    pipeline: List[StepDescription]


@dataclass
class ConfigurationFile:
    config_dict: dict[str, dict]

    def __init__(self, config_dict):
        self.config_dict = config_dict

    @staticmethod
    def parse(data: str) -> 'ConfigurationFile':
        return ConfigurationFile(config_dict=toml.loads(data))

    @staticmethod
    def load(fname: str) -> 'ConfigurationFile':
        with open(fname) as f:
            return ConfigurationFile.parse(f.read())
"""