# This is a sample Python script.

# Press Umschalt+F10 to execute it or replace it with your code.
# Press Double Shift to search everywhere for classes, files, tool windows, actions, and settings.
import multiprocessing
import os
import sys

from dataclasses import dataclass

import docopt
import toml
from loguru import logger
from tqdm import tqdm

import regioneval.utility
from paperboat.configuration.presets import default_ocr
from paperboat.model_loader import LocalModelLibrary
from paperboat.processing import PipelineBuilder
from segmentation.postprocessing.data_classes import PredictionResult
# import ray

# Do not remove these imports, these are necessary for the steps to be found
from paperboat.pipeline.preprocessing import *
from paperboat.pipeline.segmentation import *
from paperboat.pipeline.ocr import *
from paperboat.pipeline.output import *

@dataclass
class BaselinePredictorResult:
    prediction: PredictionResult
    scaled_image: SourceImage


class BaselinePredictionActor:
    def __init__(self, settings: 'PredictionSettings'):
        #if not model_path or (model_weights is not None):
        #    raise NotImplementedError("Loading model by weights is not implemented")
        self.settings = settings
        self.predictor = Predictor(settings)
        logger.info("Loaded baseline model successfully")

    def process(self, img: SourceImage) -> BaselinePredictorResult:
        prediction, scaled_image = self.predictor.predict_image(img)
        #, process_pool=multiprocessing.Pool()) # TODO: should this be parallelized???
        # maybe add this parallelization to the settings?
        return BaselinePredictorResult(prediction, scaled_image)

@dataclass
class LayoutProcessingResult:
    analyzed_content: 'AnalyzedContent'

class LayoutProcessingActor:
    def __init__(self, layout_settings: LayoutProcessingSettings):
        self.settings = layout_settings

    def process(self, bl_pred: BaselinePredictorResult) -> LayoutProcessingResult:
        # TODO: should we use innner parallelism here?
        analyzed_content = process_layout(bl_pred.prediction, bl_pred.scaled_image, multiprocessing.pool.ThreadPool(1), self.settings)
        return LayoutProcessingResult(analyzed_content)



def test_pipeline(img_f):
    # run segmentation on that image
    settings = PredictionSettings(["/home/norbert/share/BaselineModEval/mod/model_136.torch"], 1000000, None, None)
    nn_predictor = Predictor(settings)
    logger.info("Model loaded successfully!")

    with regioneval.utility.Stopwatch("Prediction"):
        prediction, scaled_image = nn_predictor.predict_image(SourceImage.load(img_f), process_pool=multiprocessing.Pool())

    scale_factor = prediction.prediction_scale_factor
    layout_settings = LayoutProcessingSettings()

    with regioneval.utility.Stopwatch("Layout"):
        analyzed_content = process_layout(prediction, scaled_image, multiprocessing.Pool(), layout_settings)

    # layout_debugging(args, analyzed_content, scaled_image, file)

    # convert this back to the original image space
    analyzed_content = analyzed_content.to_pagexml_space(scale_factor)

    # run calamari ocr


USAGE = """paperboat.
Fully automated Page Segmentation and OCR.

Usage:
    paperboat [options] -- <image_files>...
    paperboat binarize [options] <image_files>...
    paperboat segment [options] <image_files>...
    paperbaot ocr [options] <image_files>...
    paperboat load-models [--clear]
    
    
Options:
    -o <dir>, --output_dir <dir>    Set output directory
    -l <dir>, --lookup_dir <dir>    Set directory to look for auxiliary files (e.g. PageXML Files)
    -c <config>, --config <config>  Set file containing configuration options
    -p <preset>, --preset <preset>  Load a given preset
    -t <threads>                    Run on a given number of threads
    --no_gpu                        Disable GPU for Segmentation and OCR
    -v                              Verbose mode ( do not suppress output )
"""


def main(args: dict):
    if args["--no_gpu"]:
        # disable GPU as quickly as possible
        os.environ["CUDA_VISIBLE_DEVICES"] = ""

    if args["load-models"]:
        lmod = LocalModelLibrary()
        if args["--clear"]:
            lmod.clear()

        else:
            lmod.load_default()
        sys.exit(0)
    elif args["binarize"] or args["segment"] or args["ocr"]:
        logger.error("Right now, only the full pipeline is implemented")
        sys.exit(1)
        if not args["-o"]:
            logger.error("Speify an output directory using -o DIR")
            sys.exit(1)
        pb = PipelineBuilder.from_configuration(toml.loads(default_ocr))
        pb.add(PageXMLOutputToDir(output_dir=args["-o"]))
        pipe = pb.get_pipeline()
        pipe.run(args["<image_files>"][0])
    else:
        if not args["--output_dir"]:
            logger.error("Speify an output directory using -o DIR")
            sys.exit(1)
        config = toml.loads(default_ocr)
        del config["system"]["cuda_devices"]
        pb = PipelineBuilder.from_configuration(config)
        pb.add(PageXMLOutputToDir(output_dir=args["--output_dir"][0]))
        pipe = pb.get_pipeline()
        for f in tqdm(args["<image_files>"]):
            if not args["-v"]:
                pipe.run_suppressed(f)
            else:
                pipe.run(f)
        pipe.metrics.print_metrics()

    # run the default pipeline
if __name__ == '__main__':
    args = docopt.docopt(USAGE,version="0.0.1")
    main(args)


