import os
from pathlib import Path

import requests
from loguru import logger


class LocalModelLibrary:
    MODEL_SOURCE_REPO = "https://segmentation-experiments.informatik.uni-wuerzburg.de/models-deploy/"
    MODEL_FLIST = ["camgen1.ckpt.h5", "camgen1.json", "model_136.json", "model_136.torch", "uw3-me-0.ckpt.json",
                   "uw3-me-0.ckpt.h5"]

    def __init__(self):
        self.path = Path.home() / ".cache" / "paperboat_docpipe" / "models"

    def exists(self):
        return self.path.exists()

    def download_model(self, output_fname: Path, mod: str):
        logger.info(f"Downloading model {mod} from repo to {self.path}...")
        res = requests.get(LocalModelLibrary.MODEL_SOURCE_REPO + mod)
        with open(output_fname, "wb") as of:
            of.write(res.content)

    def load_default(self):
        if not self.exists():
            os.makedirs(self.path, exist_ok=True)

        downloaded_anything = False
        for f in LocalModelLibrary.MODEL_FLIST:
            fpath = self.path / f
            if not fpath.exists():
                self.download_model(output_fname=fpath, mod=f)
                downloaded_anything = True

        if downloaded_anything:
            logger.info(f"All models fetched successfully")

    def get_segmentation_model(self):
        return self.path / "model_136.torch"

    def get_ocr_model(self):
        return self.path / "uw3-me-0.ckpt.json"

    def get(self, model_name: str) -> Path:
        return self.path / model_name
    def clear(self):
        # clear the local repository
        # TODO: this is unsafe, add user confirmation
        for m in LocalModelLibrary.MODEL_FLIST:
            fpath = self.path / m
            if fpath.exists():
                logger.info(f"Deleting {fpath}")
                os.remove(fpath)
        logger.info(f"Models successfully cleared")


class ModelRetriever:
    def __init__(self, config: dict):
        logger.warning("Skipping model configuration, loading default models")
        self.model_lib = LocalModelLibrary()
        self.model_lib.load_default()
        self.config = config

    def get_lib(self) -> LocalModelLibrary:
        return self.model_lib


