from dataclasses import dataclass
from pathlib import Path

import typing

from paperboat.processing import CarryOnData, Step
from regioneval.libpagexml2 import PageXMLPage, PageXMLBuilder
from segmentation.preprocessing.source_image import SourceImage


@dataclass
class PageXMLOutput:
    image: SourceImage
    page: PageXMLPage
    pagexml: str



class PageXMLOutputToDir(Step):
    def __init__(self, output_dir):
        self.output_dir = Path(output_dir)

    def input_format(self) -> typing.Any:
        return CarryOnData # we accept filenames as string

    def output_format(self) -> typing.Any:
        return PageXMLOutput

    @classmethod
    def from_config(cls, config: dict) -> 'PageXMLOutputToDir':
        if not "output_dir" in config:
            raise KeyError("Must have configuration option output_dir")
        return PageXMLOutputToDir(config["output_dir"])

    def run(self, data: CarryOnData) -> PageXMLOutput:
        # load the image
        page: PageXMLPage = data.get("calamari.page_out")
        pagexml = PageXMLBuilder.build_pagexml(page)
        # write that to the directory
        with open((self.output_dir / page.image_filename).with_suffix(".xml"), "w") as f:
            f.write(pagexml)
        return PageXMLOutput(image=None, page=page, pagexml=pagexml)


    @classmethod
    def get_id(cls) -> str:
        return "pagexmloutputtodir"


