import imageio
import typing

from paperboat.processing import Step, CarryOnData
from segmentation.preprocessing.source_image import SourceImage


class ImageLoader(Step):
    def __init__(self):
        pass

    def input_format(self) -> typing.Any:
        return str # we accept filenames as string

    def output_format(self) -> typing.Any:
        return CarryOnData

    @classmethod
    def from_config(cls, config: dict) -> 'ImageLoader':
        return ImageLoader()

    def run(self, data: str) -> CarryOnData:
        # load the image
        img = imageio.imread(data)
        cod = CarryOnData()
        cod.set("src.filename",data)
        cod.set("src.img",img)
        return cod

    @classmethod
    def get_id(cls) -> str:
        return "imageloader"


class SourceImageizer(Step):

    @classmethod
    def get_id(cls) -> str:
        return "source_imageizer"

    def run(self, data: CarryOnData) -> CarryOnData:
        return data.set("source_image",SourceImage.from_numpy(data.get("src.img")))

    def input_format(self) -> typing.Any:
        return CarryOnData

    def output_format(self) -> typing.Any:
        return CarryOnData

    @classmethod
    def from_config(cls, config: dict) -> 'Step':
        return SourceImageizer()
